﻿Imports DFU_Programmer.USB

Public Class MainForm

    Public WithEvents USBCLIENT As New HostClient

    Private FwHexName As String
    Private FwHexBin() As Byte = Nothing

    Private Delegate Sub cbUpdateDfuStatusBar(percent As Integer)
    Private Delegate Sub cbSetStatus(msg As String)
    Private Delegate Sub cbSetConnectionStatus()
    Private Delegate Sub cbEnableLoadButton()
    Private Delegate Sub cbDisconnectEvent()

    Public AppIsClosing As Boolean = False

    Private WithEvents MAIN_FCUSB As FCUSB_DEVICE = Nothing

    Private Sub MainForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        lblAvrFn.Text = "File: no file currently loaded"
        lblAvrRange.Text = "Range: 0x0000-0x0000"
        lblAvrCrc.Text = "CRC: 0x000000"
        FwHexBin = Nothing
        FwHexName = ""
        cmdAvrProg.Enabled = False
        cmdAvrStart.Enabled = False
        cmdAvrLoad.Enabled = False
        SetConnectionStatus()
        USBCLIENT.StartService()
    End Sub

    Private Sub SetStatus(msg As String)
        If Me.InvokeRequired Then
            Dim d As New cbSetStatus(AddressOf SetStatus)
            Me.Invoke(d, New Object() {[msg]})
        Else
            Me.lblStatus.Text = msg
            Application.DoEvents()
        End If
    End Sub

    Public Sub OnDeviceUpdateProgress(device As FCUSB_DEVICE, percent As Integer) Handles MAIN_FCUSB.OnUpdateProgress
        UpdateProgress(percent)
    End Sub

    Private Sub cmdAvrLoad_Click(sender As Object, e As EventArgs) Handles cmdAvrLoad.Click
        Dim OpenMe As New OpenFileDialog
        OpenMe.AddExtension = True
        OpenMe.InitialDirectory = Application.StartupPath & "\Firmware"
        OpenMe.Title = "Choose firmware to program"
        OpenMe.CheckPathExists = True
        If MAIN_FCUSB.USBHANDLE.UsbRegistryInfo.Vid = &H3EB AndAlso MAIN_FCUSB.USBHANDLE.UsbRegistryInfo.Pid = &H2FF9 Then
            OpenMe.Filter = "Intel Hex Format (*.hex)|*XPORT*.hex"
        ElseIf MAIN_FCUSB.USBHANDLE.UsbRegistryInfo.Vid = &H3EB AndAlso MAIN_FCUSB.USBHANDLE.UsbRegistryInfo.Pid = &H2FF0 Then
            OpenMe.Filter = "Intel Hex Format (*.hex)|FCUSB.CLASSIC.*.U2.hex"
        ElseIf MAIN_FCUSB.USBHANDLE.UsbRegistryInfo.Vid = &H3EB AndAlso MAIN_FCUSB.USBHANDLE.UsbRegistryInfo.Pid = &H2FF4 Then
            OpenMe.Filter = "Intel Hex Format (*.hex)|FCUSB.CLASSIC.*.U4.hex"
        End If
        If OpenMe.ShowDialog = DialogResult.OK Then
            Dim finfo As New IO.FileInfo(OpenMe.FileName)
            Dim ihex_tool As New IHEX.StreamReader(finfo.OpenText)
            If ihex_tool.IsValid Then
                ReDim FwHexBin(ihex_tool.Length - 1)
                ihex_tool.Read(FwHexBin, 0, FwHexBin.Length)
                FwHexName = finfo.Name
                LoadHexFileInfo()
            Else
                SetStatus("Error: file is corrupt or not a AVR Hex file")
            End If
        End If
    End Sub

    Private Sub cmdAvrProg_Click(sender As Object, e As EventArgs) Handles cmdAvrProg.Click
        Try
            MAIN_FCUSB.DFU_IF.DeviceInit()
            cmdAvrLoad.Enabled = False
            cmdAvrProg.Enabled = False
            Dim DfuSize As Integer = MAIN_FCUSB.DFU_IF.DeviceSize()
            If (DfuSize = 0) Then
                SetStatus("Device is no longer in DFU mode") : Exit Sub
            End If
            If (FwHexBin.Length > DfuSize) Then
                SetStatus("Error: failed to retrieve board firmware version") : Exit Sub
            End If
            UpdateProgress(0)
            SetStatus("Programming new firmware over USB")
            If (Not MAIN_FCUSB.DFU_IF.EraseDevice()) Then
                SetStatus("Error: device erase was not successful") : Exit Sub
            End If
            Application.DoEvents()
            Threading.Thread.Sleep(250)
            Application.DoEvents()
            Threading.Thread.Sleep(250)
            MAIN_FCUSB.DFU_IF.WriteData(0, FwHexBin)
            SetStatus("New firmware programmed into USB device")
            Application.DoEvents()
            Threading.Thread.Sleep(250)
            cmdAvrStart.Enabled = True
        Catch ex As Exception
        Finally
            UpdateProgress(0)
        End Try
    End Sub

    Private Sub cmdAvrStart_Click(sender As Object, e As EventArgs) Handles cmdAvrStart.Click
        cmdAvrLoad.Enabled = False
        cmdAvrProg.Enabled = False
        cmdAvrStart.Enabled = False
        MAIN_FCUSB.DFU_IF.RunApp() 'Start application (hardware reset)
    End Sub
    'Loads the gui information and loads up the hex editor
    Private Sub LoadHexFileInfo()
        lblAvrFn.Text = String.Format("File: {0}", FwHexName)
        lblAvrRange.Text = "Range: 0x0000-0x" & Hex(FwHexBin.Length - 1).PadLeft(4, CChar("0"))
        Dim crc As Int32
        Dim i As Integer
        For i = 0 To FwHexBin.Length - 1
            crc += FwHexBin(0)
        Next
        crc = crc Xor &HFFFFFF
        crc = crc + 1
        lblAvrCrc.Text = "CRC: 0x" & Hex(crc And &HFFFFFF)
        HexViewer.CreateHexViewer(0, FwHexBin)
        cmdAvrProg.Enabled = True
    End Sub

    Private Sub UpdateProgress(percent As Integer)
        If Me.InvokeRequired Then
            Dim d As New cbUpdateDfuStatusBar(AddressOf UpdateProgress)
            Me.Invoke(d, New Object() {percent})
        Else
            If (percent > 100) Then percent = 100
            If (percent = 100) Then
                Me.statuspage_progress.Value = 0
            Else
                Me.statuspage_progress.Value = percent
            End If
            Application.DoEvents()
        End If
    End Sub

    Private Sub OnUsbDevice_Connected(usb_dev As FCUSB_DEVICE) Handles USBCLIENT.DeviceConnected
        If MAIN_FCUSB Is Nothing Then
            MAIN_FCUSB = usb_dev
        ElseIf MAIN_FCUSB Is usb_dev Then
        Else
            Exit Sub
        End If
        usb_dev.DFU_IF.DeviceInit()
        SetConnectionStatus()
        AddHandler usb_dev.DFU_IF.SetProgress, AddressOf UpdateProgress
        EnableLoadButton()
    End Sub

    Private Sub OnUsbDevice_Disconnected(usb_dev As FCUSB_DEVICE) Handles USBCLIENT.DeviceDisconnected
        DisconnectEvent() 'Thread-safe
    End Sub

    Private Sub DisconnectEvent()
        If Me.InvokeRequired Then
            Dim n As New cbDisconnectEvent(AddressOf DisconnectEvent)
            Me.Invoke(n)
        Else
            MAIN_FCUSB = Nothing
            SetConnectionStatus()
            cmdAvrLoad.Enabled = False
            cmdAvrProg.Enabled = False
            cmdAvrStart.Enabled = False
        End If
    End Sub

    Public Sub SetConnectionStatus()
        If Me.InvokeRequired Then
            Dim d As New cbSetConnectionStatus(AddressOf SetConnectionStatus)
            Me.Invoke(d)
        Else
            Try
                If AppIsClosing Then Exit Sub
                If (MAIN_FCUSB Is Nothing) Then
                    statuspage_progress.Value = 0
                    Me.lblStatus.Text = "Status: Disconnected"
                Else
                    Me.lblStatus.Text = "Status: Connected"
                End If
                Application.DoEvents()
            Catch ex As Exception
            End Try
        End If
    End Sub

    Private Sub MainForm_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        AppIsClosing = True
        USBCLIENT.CloseService = True
    End Sub

    Private Sub EnableLoadButton()
        If Me.InvokeRequired Then
            Dim n As New cbEnableLoadButton(AddressOf EnableLoadButton)
            Me.Invoke(n)
        Else
            cmdAvrLoad.Enabled = True
        End If
    End Sub


End Class
